@@ -9,7 +9,7 @@ from typing import TYPE_CHECKING, Any
 import numpy as np
 
 import blosc2
-from blosc2.ndarray import get_intersecting_chunks, npvecdot, slice_to_chunktuple
+from blosc2.ndarray import get_intersecting_chunks, nptranspose, npvecdot, slice_to_chunktuple
 
 if TYPE_CHECKING:
     from collections.abc import Sequence
@@ -79,9 +79,8 @@ def matmul(x1: blosc2.Array, x2: blosc2.NDArray, **kwargs: Any) -> blosc2.NDArra
     if np.isscalar(x1) or np.isscalar(x2):
         raise ValueError("Arguments can't be scalars.")
 
-    # Added this to pass array-api tests (which use internal getitem to check results)
-    x1 = blosc2.asarray(x1)
-    x2 = blosc2.asarray(x2)
+    # Makes a SimpleProxy if inputs are not blosc2 arrays
+    x1, x2 = blosc2.as_simpleproxy(x1, x2)
 
     # Validate matrix multiplication compatibility
     if x1.shape[builtins.max(-1, -len(x2.shape))] != x2.shape[builtins.max(-2, -len(x2.shape))]:
@@ -100,7 +99,7 @@ def matmul(x1: blosc2.Array, x2: blosc2.NDArray, **kwargs: Any) -> blosc2.NDArra
     n, k = x1.shape[-2:]
     m = x2.shape[-1]
     result_shape = np.broadcast_shapes(x1.shape[:-2], x2.shape[:-2]) + (n, m)
-    result = blosc2.zeros(result_shape, dtype=np.result_type(x1, x2), **kwargs)
+    result = blosc2.zeros(result_shape, dtype=blosc2.result_type(x1, x2), **kwargs)
 
     if 0 not in result.shape + x1.shape + x2.shape:  # if any array is empty, return array of 0s
         p, q = result.chunks[-2:]
@@ -128,9 +127,9 @@ def matmul(x1: blosc2.Array, x2: blosc2.NDArray, **kwargs: Any) -> blosc2.NDArra
                         result[chunk + (slice(row, row_end), slice(col, col_end))] += np.matmul(bx1, bx2)
 
     if x1_is_vector:
-        result.squeeze(axis=-2)
+        result = result.squeeze(axis=-2)
     if x2_is_vector:
-        result.squeeze(axis=-1)
+        result = result.squeeze(axis=-1)
 
     return result
 
@@ -183,11 +182,9 @@ def tensordot(
     """
     fast_path = kwargs.pop("fast_path", None)  # for testing purposes
     # TODO: add fast path for when don't need to change chunkshapes
-    # Added this to pass array-api tests (which use internal getitem to check results)
-    if isinstance(x1, np.ndarray) and isinstance(x2, np.ndarray):
-        return np.tensordot(x1, x2, axes=axes)
 
-    x1, x2 = blosc2.asarray(x1), blosc2.asarray(x2)
+    # Makes a SimpleProxy if inputs are not blosc2 arrays
+    x1, x2 = blosc2.as_simpleproxy(x1, x2)
 
     if isinstance(axes, tuple):
         a_axes, b_axes = axes
@@ -227,7 +224,7 @@ def tensordot(
         raise ValueError("x1 and x2 must have same shapes along reduction dimensions")
 
     result_shape = tuple(x1shape[a_keep]) + tuple(x2shape[b_keep])
-    result = blosc2.zeros(result_shape, dtype=np.result_type(x1, x2), **kwargs)
+    result = blosc2.zeros(result_shape, dtype=blosc2.result_type(x1, x2), **kwargs)
 
     op_chunks = [
         slice_to_chunktuple(slice(0, s, 1), c) for s, c in zip(x1shape[a_axes], a_chunks_red, strict=True)
@@ -261,24 +258,8 @@ def tensordot(
         a_selection = tuple(next(rchunk_iter) if a else slice(None, None, 1) for a in a_keep)
         b_selection = tuple(next(rchunk_iter) if b else slice(None, None, 1) for b in b_keep)
         res_chunks = tuple(s.stop - s.start for s in res_chunk)
-
-        if fast_path:  # just load everything
-            bx1 = x1[a_selection]
-            bx2 = x2[b_selection]
-            newshape_a = (
-                math.prod([bx1.shape[i] for i in a_keep_axes]),
-                math.prod([bx1.shape[a] for a in a_axes]),
-            )
-            newshape_b = (
-                math.prod([bx2.shape[b] for b in b_axes]),
-                math.prod([bx2.shape[i] for i in b_keep_axes]),
-            )
-            at = bx1.transpose(newaxes_a).reshape(newshape_a)
-            bt = bx2.transpose(newaxes_b).reshape(newshape_b)
-            res = np.dot(at, bt)
-            result[res_chunk] += res.reshape(res_chunks)
-        else:  # operands too big, have to go chunk-by-chunk
-            for ochunk in product(*op_chunks):
+        for ochunk in product(*op_chunks):
+            if not fast_path:  # operands too big, have to go chunk-by-chunk
                 op_chunk = tuple(
                     slice(rc * rcs, builtins.min((rc + 1) * rcs, x1s), 1)
                     for rc, rcs, x1s in zip(ochunk, a_chunks_red, a_shape_red, strict=True)
@@ -293,21 +274,23 @@ def tensordot(
                     op_chunk[next(order_iter)] if not b else bs_
                     for bs_, b in zip(b_selection, b_keep, strict=True)
                 )
-                bx1 = x1[a_selection]
-                bx2 = x2[b_selection]
-                # adapted from numpy tensordot
-                newshape_a = (
-                    math.prod([bx1.shape[i] for i in a_keep_axes]),
-                    math.prod([bx1.shape[a] for a in a_axes]),
-                )
-                newshape_b = (
-                    math.prod([bx2.shape[b] for b in b_axes]),
-                    math.prod([bx2.shape[i] for i in b_keep_axes]),
-                )
-                at = bx1.transpose(newaxes_a).reshape(newshape_a)
-                bt = bx2.transpose(newaxes_b).reshape(newshape_b)
-                res = np.dot(at, bt)
-                result[res_chunk] += res.reshape(res_chunks)
+            bx1 = x1[a_selection]
+            bx2 = x2[b_selection]
+            # adapted from numpy tensordot
+            newshape_a = (
+                math.prod([bx1.shape[i] for i in a_keep_axes]),
+                math.prod([bx1.shape[a] for a in a_axes]),
+            )
+            newshape_b = (
+                math.prod([bx2.shape[b] for b in b_axes]),
+                math.prod([bx2.shape[i] for i in b_keep_axes]),
+            )
+            at = nptranspose(bx1, newaxes_a).reshape(newshape_a)
+            bt = nptranspose(bx2, newaxes_b).reshape(newshape_b)
+            res = np.dot(at, bt)
+            result[res_chunk] += res.reshape(res_chunks)
+            if fast_path:  # already done everything
+                break
     return result
 
 
@@ -342,7 +325,8 @@ def vecdot(x1: blosc2.NDArray, x2: blosc2.NDArray, axis: int = -1, **kwargs) ->
     if isinstance(x1, np.ndarray) and isinstance(x2, np.ndarray):
         return npvecdot(x1, x2, axis=axis)
 
-    x1, x2 = blosc2.asarray(x1), blosc2.asarray(x2)
+    # Makes a SimpleProxy if inputs are not blosc2 arrays
+    x1, x2 = blosc2.as_simpleproxy(x1, x2)
 
     N = builtins.min(x1.ndim, x2.ndim)
     if axis < -N or axis > -1:
@@ -363,7 +347,7 @@ def vecdot(x1: blosc2.NDArray, x2: blosc2.NDArray, axis: int = -1, **kwargs) ->
         raise ValueError("x1 and x2 must have same shapes along reduction dimensions")
 
     result_shape = np.broadcast_shapes(x1shape[a_keep], x2shape[b_keep])
-    result = blosc2.zeros(result_shape, dtype=np.result_type(x1, x2), **kwargs)
+    result = blosc2.zeros(result_shape, dtype=blosc2.result_type(x1, x2), **kwargs)
 
     res_chunks = [
         slice_to_chunktuple(s, c)
@@ -396,19 +380,17 @@ def vecdot(x1: blosc2.NDArray, x2: blosc2.NDArray, axis: int = -1, **kwargs) ->
         )
         b_selection = tuple(next(rchunk_iter) if b else slice(None, None, 1) for b in b_keep)
 
-        if fast_path:  # just load everything, also handles case of 0 in shapes
-            bx1 = x1[a_selection]
-            bx2 = x2[b_selection]
-            result[res_chunk] += npvecdot(bx1, bx2, axis=axis)  # handles conjugation of bx1
-        else:  # operands too big, have to go chunk-by-chunk
-            for ochunk in range(0, a_shape_red, a_chunks_red):
+        for ochunk in range(0, a_shape_red, a_chunks_red):
+            if not fast_path:  # operands too big, go chunk-by-chunk
                 op_chunk = (slice(ochunk, builtins.min(ochunk + a_chunks_red, x1.shape[a_axes]), 1),)
                 a_selection = a_selection[:a_axes] + op_chunk + a_selection[a_axes + 1 :]
                 b_selection = b_selection[:b_axes] + op_chunk + b_selection[b_axes + 1 :]
-                bx1 = x1[a_selection]
-                bx2 = x2[b_selection]
-                res = npvecdot(bx1, bx2, axis=axis)  # handles conjugation of bx1
-                result[res_chunk] += res
+            bx1 = x1[a_selection]
+            bx2 = x2[b_selection]
+            res = npvecdot(bx1, bx2, axis=axis)  # handles conjugation of bx1
+            result[res_chunk] += res
+            if fast_path:  # already done everything
+                break
     return result
 
 
@@ -486,8 +468,9 @@ def permute_dims(
     """
     if np.isscalar(arr) or arr.ndim < 2:
         return arr
-    if isinstance(arr, np.ndarray):  # for array-api test compliance (does getitem for comparison)
-        return np.permute_dims(arr, axes)
+
+    # Makes a SimpleProxy if input is not blosc2 array
+    arr = blosc2.as_simpleproxy(arr)
 
     ndim = arr.ndim
 
@@ -506,9 +489,15 @@ def permute_dims(
 
     chunks = arr.chunks
     shape = arr.shape
-
-    for info in arr.iterchunks_info():
-        coords = info.coords
+    # handle SimpleProxy which doesn't have iterchunks_info
+    if hasattr(arr, "iterchunks_info"):
+        my_it = arr.iterchunks_info()
+        _get_el = lambda x: x.coords  # noqa: E731
+    else:
+        my_it = get_intersecting_chunks((), shape, chunks)
+        _get_el = lambda x: x.raw  # noqa: E731
+    for info in my_it:
+        coords = _get_el(info)
         start_stop = [
             (coord * chunk, builtins.min(chunk * (coord + 1), dim))
             for coord, chunk, dim in zip(coords, chunks, shape, strict=False)
@@ -517,7 +506,7 @@ def permute_dims(
         src_slice = tuple(slice(start, stop) for start, stop in start_stop)
         dst_slice = tuple(slice(start_stop[ax][0], start_stop[ax][1]) for ax in axes)
 
-        transposed = np.transpose(arr[src_slice], axes=axes)
+        transposed = nptranspose(arr[src_slice], axes=axes)
         result[dst_slice] = np.ascontiguousarray(transposed)
 
     return result
@@ -555,7 +544,8 @@ def transpose(x, **kwargs: Any) -> blosc2.NDArray:
     # If arguments are dimension < 2, they are returned
     if np.isscalar(x) or x.ndim < 2:
         return x
-
+    # Makes a SimpleProxy if input is not blosc2 array
+    x = blosc2.as_simpleproxy(x)
     # Validate arguments are dimension 2
     if x.ndim > 2:
         raise ValueError("Transposing arrays with dimension greater than 2 is not supported yet.")
@@ -579,6 +569,8 @@ def matrix_transpose(arr: blosc2.Array, **kwargs: Any) -> blosc2.NDArray:
         ``(..., N, M)``.
     """
     axes = None
+    # Makes a SimpleProxy if input is not blosc2 array
+    arr = blosc2.as_simpleproxy(arr)
     if not np.isscalar(arr) and arr.ndim > 2:
         axes = list(range(arr.ndim))
         axes[-2], axes[-1] = axes[-1], axes[-2]
@@ -612,6 +604,8 @@ def diagonal(x: blosc2.blosc2.NDArray, offset: int = 0) -> blosc2.blosc2.NDArray
 
     Reference: https://data-apis.org/array-api/latest/extensions/generated/array_api.linalg.diag.html#diag
     """
+    # Makes a SimpleProxy if input is not blosc2 array
+    x = blosc2.as_simpleproxy(x)
     n_rows, n_cols = x.shape[-2:]
     min_idx = builtins.min(n_rows, n_cols)
     if offset < 0:
@@ -648,6 +642,7 @@ def outer(x1: blosc2.blosc2.NDArray, x2: blosc2.blosc2.NDArray, **kwargs: Any) -
     out: blosc2.NDArray
         A two-dimensional array containing the outer product and whose shape is (N, M).
     """
+    x1, x2 = blosc2.as_simpleproxy(x1, x2)
     if (x1.ndim != 1) or (x2.ndim != 1):
         raise ValueError("outer only valid for 1D inputs.")
     return tensordot(x1, x2, ((), ()), **kwargs)  # for testing purposes
